/**********************************************************************
 ** Namespace fnp: file name processing
 **********************************************************************/


#if !defined(FNP_H)
#define FNP_H 1

#include <vector>
#include <string>
#include <map>


namespace stdi {

  using namespace std;
  
  typedef vector<string> StringArray;
  
  // File/name parser
  namespace fnp {
    
    vector<string> regularFilesInDir(const string & dname);
    vector<string> regularFilesInDir(const char * dname);
    vector<string> regularFilesInDir(const string & dname, const char *re);
    vector<string> regularFilesInDir(const char * dname, const char *re);
    vector<string> subDirsInDir(const string & dname );
    vector<string> subDirsInDir(const char *  dname );
    
    /*----------------------------------------------------------------- 
      Example of parsing a pathname
      pathname = /home/media/user/check.txt
      root = /home/media/user/check
      extension = txt
      head =  /home/media/user
      tail = check.txt
      -----------------------------------------------------------------*/
    
    
    string extension(const string & fname);
    string extension(const char * fname);
    
    string root(const string & fname);
    string root(const char * fname);
    
    string head(const string & fname);
    string head(const char * fname);
    
    string tail(const string & fname);
    string tail(const char * fname);

    // File info & basic operations    
    bool exists(const string & fname);
    bool exists(const char * fname);

    bool isDirectory(const string & fname);
    bool isDirectory(const char * fname);

    bool isRegularFile(const string & fname);
    bool isRegularFile(const char * fname);

    
    bool isRegularFile(const string &dir, const string & fname);
    bool isRegularFile(const char *dir, const char * fname);

    bool createDirectory(const string &dname);

    // Full pathname
    string fullname(const string &dir, const string fname);
    string fullname(const char *dir, const char *fname);
    
    string clean(const string &name);
    string clean(const char *name);
    
    
    // Text parsers
    vector<StringArray> parseTextFile(const string &filename, 
				      const char *seps=" \t");
    StringArray parseLine(const string &s, const char *seps=" \t");
    string copySubString(const string &s, int b, int e);

  } // End of namespace fnp
  
  typedef map<string, string, less<string> > ConfigureMap;
  
  class Configure: private  ConfigureMap {
    
  public:
    
    Configure();
    Configure(const string &fname);
    virtual ~Configure();
    
    string property(const string &key);
    
  };
  
  namespace file = fnp;

} // End of namespace stdi

#endif // FNP_H


