#include "fnp.h"
using namespace stdi;

#include <iostream>
#include <fstream>
using namespace std;

#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fnmatch.h>
#include <unistd.h>


#define  FILE_MODE  (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
#define  DIR_MODE   (FILE_MODE | S_IXUSR | S_IXGRP | S_IXOTH)


bool fnp::createDirectory(const string &dname)
{
  return mkdir(dname.c_str(), DIR_MODE)>=0;
}


bool fnp::exists(const string & fname)
{
  return access(fname.c_str(), F_OK) >= 0;
}

bool fnp::exists(const char * fname)
{
  return access(fname, F_OK) >= 0;
}

bool fnp::isDirectory(const string & fname)
{
  return fnp::isDirectory(fname.c_str());
}

bool fnp::isDirectory(const char * fname)
{
  if (access(fname, F_OK)<0) return false;
  struct stat  buf;
  if (stat(fname, &buf) < 0) {
    cerr << "Waring: stat error" << endl;
    exit(1);
  }
  if (S_ISDIR(buf.st_mode)) return true;
  return false;
}

vector<string> fnp::regularFilesInDir(const string & dname)
{
  return regularFilesInDir(dname.c_str());
}

vector<string> fnp::regularFilesInDir(const char * _dname)
{
  const string __dname ( fnp::clean(_dname) );
  const char *dname = (const char *)__dname.c_str();

  DIR  *dp;
  struct dirent   *dirp;
  if ( (dp = opendir(dname)) == NULL) {
    cerr << "can't open " << dname << endl;
    return vector<string>();
  }

  vector<string> files;
  while ( (dirp = readdir(dp)) != NULL) 
    if (isRegularFile(dname, dirp->d_name)) 
      files.push_back(dirp->d_name);
  closedir(dp);

  return files;
}

vector<string> fnp::regularFilesInDir(const string & dname, const char *re)
{
  return regularFilesInDir(dname.c_str(), re);
}

vector<string> fnp::regularFilesInDir(const char * _dname, const char *re)
{
  const string __dname ( fnp::clean(_dname) );
  const char *dname = (const char *)__dname.c_str();

  DIR  *dp;
  struct dirent   *dirp;
 
  if ( (dp = opendir(dname)) == NULL) {
    cerr << "can't open " << dname << endl;
    return vector<string>();;
  }

  vector<string> files;
  while ( (dirp = readdir(dp)) != NULL) 
    if (fnmatch(re,  dirp->d_name, FNM_PATHNAME)==0)
      if (isRegularFile(dname, dirp->d_name)) 
	files.push_back(dirp->d_name);
  closedir(dp);

  return files;
}

vector<string> fnp::subDirsInDir(const string& dname){
    return subDirsInDir(dname.c_str());
}

vector<string> fnp::subDirsInDir(const char * _dname ){
    
    const string __dname ( fnp::clean(_dname) );
    const char *dname = (const char *)__dname.c_str();
    
    DIR  *dp;
    struct dirent   *dirp;
    if ( (dp = opendir(dname)) == NULL) {
	cerr << "can't open " << dname << endl;
	return vector<string>();
    }
    
    vector<string> dirs;
    /*
    while ( (dirp = readdir(dp)) != NULL) 
	if (isRegularFile(dname, dirp->d_name)) 
	    files.push_back(dirp->d_name);
    */
    while( (dirp = readdir(dp)) != NULL ){
	
	string fname = string(dname) + "/" +string(dirp->d_name);
	if( isDirectory( fname ) && string(dirp->d_name) != "." && string(dirp->d_name) != ".." )
	    dirs.push_back(dirp->d_name);
    }
	
    closedir(dp);
    
    return dirs;
}

string fnp::extension(const string & fname)
{
  return extension(fname.c_str());
}

string fnp::extension(const char * fname)
{
  if (!fname) return string();
  int n = strlen(fname);
  if (n < 1) return string();

  char * p = (char *) (fname+n-1);
  bool found = false;

  while (*p != '/' && p >= fname) {
    if (*p == '.') {
      found = true;
      break;
    }
    p--;
  }
  if (!found) return string();
  return string(++p);
}


string fnp::root(const string & fname)
{
  return root(fname.c_str());
}


string fnp::root(const char * fname)
{
  if (!fname) return string();
  int n = strlen(fname);
  if (n < 1) return string();

  char * p = (char *) (fname+n-1);
  bool found = false;

  while (*p != '/' && p >= fname) {
    if (*p == '.') {
      found = true;
      break;
    }
    p--;
  }
  if (!found) return string(fname);

  string _fname(fname);
  return string(_fname, 0, p-fname);
}


string fnp::head(const string & fname)
{
  return head(fname.c_str());
}

string fnp::head(const char * fname)
{
  if (!fname) return string();
  int n = strlen(fname);
  if (n < 1) return string();

  char * p = (char *) (fname+n-1);
  bool found = false;

  while ( p >= fname) {
    if (*p == '/') {
      found = true;
      break;
    }
    p--;
  }

  if (!found) return string();
  string _fname(fname);
  return string(_fname, 0, p-fname);
}


string fnp::tail(const string & fname)
{
  return tail(fname.c_str());
}

string fnp::tail(const char * fname)
{
  if (!fname) return string();
  int n = strlen(fname);
  if (n < 1) return string();

  char * p = (char *) (fname+n-1);
  bool found = false;

  while ( p >= fname) {
    if (*p == '/') {
      found = true;
      break;
    }
    p--;
  }

  string _fname(fname);
  if (!found) return _fname;
  return string(_fname, p-fname+1, n);
}


bool fnp::isRegularFile(const string & fname)
{
  return isRegularFile(fname.c_str());
}

bool fnp::isRegularFile(const char * fname)
{
  if (access(fname, F_OK)<0) return false;

  struct stat  buf;
  if (stat(fname, &buf) < 0) {
    cerr << "Waring: stat error" << endl;
    exit(1);
  }
 
  if (S_ISREG(buf.st_mode)) return true;

  return false;
}

bool fnp::isRegularFile(const string &dir, const string & fname)
{
  return isRegularFile(dir+"/"+fname);
}
bool fnp::isRegularFile(const char *dir, const char * fname)
{
  return isRegularFile(string(dir), string(fname));
}

string fnp::fullname(const char *dir, const char *fname)
{
  return fnp::fullname(string(dir), string(fname));
}

string fnp::clean(const string &name)
{
  if (name.empty()) return string();
  string d(name);
  if  (*(d.end()-1) == '/') 
    d.erase(d.end()-1);

  return d;
}


string fnp::clean(const char *name)
{
  return fnp::clean(string(name));
}


string fnp::fullname(const string &dir, const string fname)
{
  string d = fnp::clean(dir);
  string f = fnp::clean(fname);

  if (!f.empty())
    if (f[0]=='/') {
      cerr << "Warning in fnp::fullname" << endl;
      cerr << "Invalid (relative) filename, return empty string" << endl;
      return string();
    }
  
  if (d.empty()) return f;
  if (!d.compare(".")) return f;
  return d + "/" + f;
}


// Get a separate deep copy of substring
string fnp::copySubString(const string &s, int b, int e) 
{
  if (s.empty()) return string();
  if (e==string::npos) e = s.size();
  if (b<0) b = 0;
  if (b>=e) return string();
  
  string t(e-b, char(0));
  for(int i=b; i<e; i++) t[i-b] = s[i];
  return t;
}


StringArray fnp::parseLine(const string &s, const char *seps) {

  if (s.empty()) return StringArray();

  StringArray line;
  int b(0), e;

  while(1) {    
    if ( (b=s.find_first_not_of(seps, b)) == string::npos) 
      return line;
    if ( (e=s.find_first_of(seps, b))==string::npos ) {
      line.push_back(copySubString(s, b, string::npos));
      return line;
    } 
    line.push_back(copySubString(s, b, e));
    b = e;  
  }  
  return line;
}


vector<StringArray> fnp::parseTextFile(const string &filename, const char *seps) {

  vector<StringArray> lines;
  const int sz ( 1024 ); 
  char buf[sz];  
  ifstream in(filename.c_str());
  if (!in) {
    cerr << "Cannot open file: " << filename << endl;
    return lines;
  }

  while(in.getline(buf, sz)) {
    StringArray line = parseLine(buf, seps);
    if (!line.empty()) lines.push_back(line);
  }
  in.close();

  return lines;
}

Configure::Configure():ConfigureMap() {}
Configure::Configure(const string &fname)
  : ConfigureMap()
{
  vector<StringArray> lines = fnp::parseTextFile(fname, " \t=");
  for(int i=0; i<lines.size(); i++)
    if (lines[i].size()==2) 
      if (lines[i][0][0]!='[' && lines[i][0][0]!='#') {	
	ConfigureMap::iterator p = this->find(lines[i][0]);
	if (p != this->end()) this->erase(p);
	this->insert(value_type(lines[i][0], lines[i][1]));	
      }
}


Configure::~Configure()
{
}

string Configure::property(const string &key)
{
  ConfigureMap::iterator p = this->find(key);
  if (p == this->end()) return string();
  return (*p).second;
}

